@@ -58,9 +58,12 @@ class RecurrentNetwork(TFModelV2):
     """
 
     @override(ModelV2)
-    def forward(self, input_dict: Dict[str, TensorType],
-                state: List[TensorType],
-                seq_lens: TensorType) -> (TensorType, List[TensorType]):
+    def forward(
+        self,
+        input_dict: Dict[str, TensorType],
+        state: List[TensorType],
+        seq_lens: TensorType,
+    ) -> (TensorType, List[TensorType]):
         """Adds time dimension to batch before sending inputs to forward_rnn().
 
         You should implement forward_rnn() in your subclass."""
@@ -68,13 +71,15 @@ class RecurrentNetwork(TFModelV2):
         padded_inputs = input_dict["obs_flat"]
         max_seq_len = tf.shape(padded_inputs)[0] // tf.shape(seq_lens)[0]
         output, new_state = self.forward_rnn(
-            add_time_dimension(
-                padded_inputs, max_seq_len=max_seq_len, framework="tf"), state,
-            seq_lens)
+            add_time_dimension(padded_inputs, max_seq_len=max_seq_len, framework="tf"),
+            state,
+            seq_lens,
+        )
         return tf.reshape(output, [-1, self.num_outputs]), new_state
 
-    def forward_rnn(self, inputs: TensorType, state: List[TensorType],
-                    seq_lens: TensorType) -> (TensorType, List[TensorType]):
+    def forward_rnn(
+        self, inputs: TensorType, state: List[TensorType], seq_lens: TensorType
+    ) -> (TensorType, List[TensorType]):
         """Call the model with the given input tensors and state.
 
         Args:
@@ -113,15 +118,20 @@ class RecurrentNetwork(TFModelV2):
 
 
 class LSTMWrapper(RecurrentNetwork):
-    """An LSTM wrapper serving as an interface for ModelV2s that set use_lstm.
-    """
+    """An LSTM wrapper serving as an interface for ModelV2s that set use_lstm."""
 
-    def __init__(self, obs_space: gym.spaces.Space,
-                 action_space: gym.spaces.Space, num_outputs: int,
-                 model_config: ModelConfigDict, name: str):
+    def __init__(
+        self,
+        obs_space: gym.spaces.Space,
+        action_space: gym.spaces.Space,
+        num_outputs: int,
+        model_config: ModelConfigDict,
+        name: str,
+    ):
 
-        super(LSTMWrapper, self).__init__(obs_space, action_space, None,
-                                          model_config, name)
+        super(LSTMWrapper, self).__init__(
+            obs_space, action_space, None, model_config, name
+        )
         # At this point, self.num_outputs is the number of nodes coming
         # from the wrapped (underlying) model. In other words, self.num_outputs
         # is the input size for the LSTM layer.
@@ -133,8 +143,7 @@ class LSTMWrapper(RecurrentNetwork):
         self.use_prev_action = model_config["lstm_use_prev_action"]
         self.use_prev_reward = model_config["lstm_use_prev_reward"]
 
-        self.action_space_struct = get_base_struct_from_space(
-            self.action_space)
+        self.action_space_struct = get_base_struct_from_space(self.action_space)
         self.action_dim = 0
 
         for space in tree.flatten(self.action_space_struct):
@@ -155,55 +164,58 @@ class LSTMWrapper(RecurrentNetwork):
 
         # Define input layers.
         input_layer = tf.keras.layers.Input(
-            shape=(None, self.num_outputs), name="inputs")
+            shape=(None, self.num_outputs), name="inputs"
+        )
 
         # Set self.num_outputs to the number of output nodes desired by the
         # caller of this constructor.
         self.num_outputs = num_outputs
 
-        state_in_h = tf.keras.layers.Input(shape=(self.cell_size, ), name="h")
-        state_in_c = tf.keras.layers.Input(shape=(self.cell_size, ), name="c")
+        state_in_h = tf.keras.layers.Input(shape=(self.cell_size,), name="h")
+        state_in_c = tf.keras.layers.Input(shape=(self.cell_size,), name="c")
         seq_in = tf.keras.layers.Input(shape=(), name="seq_in", dtype=tf.int32)
 
         # Preprocess observation with a hidden layer and send to LSTM cell
         lstm_out, state_h, state_c = tf.keras.layers.LSTM(
-            self.cell_size,
-            return_sequences=True,
-            return_state=True,
-            name="lstm")(
-                inputs=input_layer,
-                mask=tf.sequence_mask(seq_in),
-                initial_state=[state_in_h, state_in_c])
+            self.cell_size, return_sequences=True, return_state=True, name="lstm"
+        )(
+            inputs=input_layer,
+            mask=tf.sequence_mask(seq_in),
+            initial_state=[state_in_h, state_in_c],
+        )
 
         # Postprocess LSTM output with another hidden layer and compute values
         logits = tf.keras.layers.Dense(
-            self.num_outputs,
-            activation=tf.keras.activations.linear,
-            name="logits")(lstm_out)
-        values = tf.keras.layers.Dense(
-            1, activation=None, name="values")(lstm_out)
+            self.num_outputs, activation=tf.keras.activations.linear, name="logits"
+        )(lstm_out)
+        values = tf.keras.layers.Dense(1, activation=None, name="values")(lstm_out)
 
         # Create the RNN model
         self._rnn_model = tf.keras.Model(
             inputs=[input_layer, seq_in, state_in_h, state_in_c],
-            outputs=[logits, values, state_h, state_c])
+            outputs=[logits, values, state_h, state_c],
+        )
         # Print out model summary in INFO logging mode.
         if logger.isEnabledFor(logging.INFO):
             self._rnn_model.summary()
 
         # Add prev-a/r to this model's view, if required.
         if model_config["lstm_use_prev_action"]:
-            self.view_requirements[SampleBatch.PREV_ACTIONS] = \
-                ViewRequirement(SampleBatch.ACTIONS, space=self.action_space,
-                                shift=-1)
+            self.view_requirements[SampleBatch.PREV_ACTIONS] = ViewRequirement(
+                SampleBatch.ACTIONS, space=self.action_space, shift=-1
+            )
         if model_config["lstm_use_prev_reward"]:
-            self.view_requirements[SampleBatch.PREV_REWARDS] = \
-                ViewRequirement(SampleBatch.REWARDS, shift=-1)
+            self.view_requirements[SampleBatch.PREV_REWARDS] = ViewRequirement(
+                SampleBatch.REWARDS, shift=-1
+            )
 
     @override(RecurrentNetwork)
-    def forward(self, input_dict: Dict[str, TensorType],
-                state: List[TensorType],
-                seq_lens: TensorType) -> (TensorType, List[TensorType]):
+    def forward(
+        self,
+        input_dict: Dict[str, TensorType],
+        state: List[TensorType],
+        seq_lens: TensorType,
+    ) -> (TensorType, List[TensorType]):
         assert seq_lens is not None
         # Push obs through "unwrapped" net's `forward()` first.
         wrapped_out, _ = self._wrapped_forward(input_dict, [], None)
@@ -223,21 +235,23 @@ class LSTMWrapper(RecurrentNetwork):
                         prev_a,
                         spaces_struct=self.action_space_struct,
                         time_axis=False,
-                    ))
+                    )
+                )
             # If actions are already flattened (but not one-hot'd yet!),
             # one-hot discrete/multi-discrete actions here.
             else:
                 if isinstance(self.action_space, (Discrete, MultiDiscrete)):
                     prev_a = one_hot(prev_a, self.action_space)
                 prev_a_r.append(
-                    tf.reshape(
-                        tf.cast(prev_a, tf.float32), [-1, self.action_dim]))
+                    tf.reshape(tf.cast(prev_a, tf.float32), [-1, self.action_dim])
+                )
         # Prev rewards.
         if self.model_config["lstm_use_prev_reward"]:
             prev_a_r.append(
                 tf.reshape(
-                    tf.cast(input_dict[SampleBatch.PREV_REWARDS], tf.float32),
-                    [-1, 1]))
+                    tf.cast(input_dict[SampleBatch.PREV_REWARDS], tf.float32), [-1, 1]
+                )
+            )
 
         # Concat prev. actions + rewards to the "main" input.
         if prev_a_r:
@@ -248,10 +262,10 @@ class LSTMWrapper(RecurrentNetwork):
         return super().forward(input_dict, state, seq_lens)
 
     @override(RecurrentNetwork)
-    def forward_rnn(self, inputs: TensorType, state: List[TensorType],
-                    seq_lens: TensorType) -> (TensorType, List[TensorType]):
-        model_out, self._value_out, h, c = self._rnn_model([inputs, seq_lens] +
-                                                           state)
+    def forward_rnn(
+        self, inputs: TensorType, state: List[TensorType], seq_lens: TensorType
+    ) -> (TensorType, List[TensorType]):
+        model_out, self._value_out, h, c = self._rnn_model([inputs, seq_lens] + state)
         return model_out, [h, c]
 
     @override(ModelV2)
@@ -270,23 +284,24 @@ class Keras_LSTMWrapper(tf.keras.Model if tf else object):
     """A tf keras auto-LSTM wrapper used when `use_lstm`=True."""
 
     def __init__(
-            self,
-            input_space: gym.spaces.Space,
-            action_space: gym.spaces.Space,
-            num_outputs: Optional[int] = None,
-            *,
-            name: str,
-            wrapped_cls: Type["tf.keras.Model"],
-            max_seq_len: int = 20,
-            lstm_cell_size: int = 256,
-            lstm_use_prev_action: bool = False,
-            lstm_use_prev_reward: bool = False,
-            **kwargs,
+        self,
+        input_space: gym.spaces.Space,
+        action_space: gym.spaces.Space,
+        num_outputs: Optional[int] = None,
+        *,
+        name: str,
+        wrapped_cls: Type["tf.keras.Model"],
+        max_seq_len: int = 20,
+        lstm_cell_size: int = 256,
+        lstm_use_prev_action: bool = False,
+        lstm_use_prev_reward: bool = False,
+        **kwargs,
     ):
 
         super().__init__(name=name)
         self.wrapped_keras_model = wrapped_cls(
-            input_space, action_space, None, name="wrapped_" + name, **kwargs)
+            input_space, action_space, None, name="wrapped_" + name, **kwargs
+        )
 
         self.action_space = action_space
         self.max_seq_len = max_seq_len
@@ -299,10 +314,10 @@ class Keras_LSTMWrapper(tf.keras.Model if tf else object):
         # observation space.
         if self.wrapped_keras_model.layers:
             assert self.wrapped_keras_model.layers[-1].outputs
-            assert len(
-                self.wrapped_keras_model.layers[-1].outputs[0].shape) == 2
+            assert len(self.wrapped_keras_model.layers[-1].outputs[0].shape) == 2
             wrapped_num_outputs = int(
-                self.wrapped_keras_model.layers[-1].outputs[0].shape[1])
+                self.wrapped_keras_model.layers[-1].outputs[0].shape[1]
+            )
         else:
             wrapped_num_outputs = int(np.product(self.obs_space.shape))
 
@@ -327,73 +342,74 @@ class Keras_LSTMWrapper(tf.keras.Model if tf else object):
 
         # Define input layers.
         input_layer = tf.keras.layers.Input(
-            shape=(None, wrapped_num_outputs), name="inputs")
+            shape=(None, wrapped_num_outputs), name="inputs"
+        )
 
-        state_in_h = tf.keras.layers.Input(
-            shape=(self.lstm_cell_size, ), name="h")
-        state_in_c = tf.keras.layers.Input(
-            shape=(self.lstm_cell_size, ), name="c")
+        state_in_h = tf.keras.layers.Input(shape=(self.lstm_cell_size,), name="h")
+        state_in_c = tf.keras.layers.Input(shape=(self.lstm_cell_size,), name="c")
         seq_in = tf.keras.layers.Input(shape=(), name="seq_in", dtype=tf.int32)
 
         # Preprocess observation with a hidden layer and send to LSTM cell
         lstm_out, state_h, state_c = tf.keras.layers.LSTM(
-            self.lstm_cell_size,
-            return_sequences=True,
-            return_state=True,
-            name="lstm")(
-                inputs=input_layer,
-                mask=tf.sequence_mask(seq_in),
-                initial_state=[state_in_h, state_in_c])
+            self.lstm_cell_size, return_sequences=True, return_state=True, name="lstm"
+        )(
+            inputs=input_layer,
+            mask=tf.sequence_mask(seq_in),
+            initial_state=[state_in_h, state_in_c],
+        )
 
         # Postprocess LSTM output with another hidden layer
         # if num_outputs not None.
         if num_outputs:
             logits = tf.keras.layers.Dense(
-                num_outputs,
-                activation=tf.keras.activations.linear,
-                name="logits")(lstm_out)
+                num_outputs, activation=tf.keras.activations.linear, name="logits"
+            )(lstm_out)
         else:
             logits = lstm_out
         # Compute values.
-        values = tf.keras.layers.Dense(
-            1, activation=None, name="values")(lstm_out)
+        values = tf.keras.layers.Dense(1, activation=None, name="values")(lstm_out)
 
         # Create the RNN model
         self._rnn_model = tf.keras.Model(
             inputs=[input_layer, seq_in, state_in_h, state_in_c],
-            outputs=[logits, values, state_h, state_c])
+            outputs=[logits, values, state_h, state_c],
+        )
 
         # Use view-requirements of wrapped model and add own
         # requirements.
-        self.view_requirements = \
-            getattr(self.wrapped_keras_model, "view_requirements", {
-                SampleBatch.OBS: ViewRequirement(space=input_space)
-            })
+        self.view_requirements = getattr(
+            self.wrapped_keras_model,
+            "view_requirements",
+            {SampleBatch.OBS: ViewRequirement(space=input_space)},
+        )
 
         # Add prev-a/r to this model's view, if required.
         if self.lstm_use_prev_action:
-            self.view_requirements[SampleBatch.PREV_ACTIONS] = \
-                ViewRequirement(SampleBatch.ACTIONS, space=self.action_space,
-                                shift=-1)
+            self.view_requirements[SampleBatch.PREV_ACTIONS] = ViewRequirement(
+                SampleBatch.ACTIONS, space=self.action_space, shift=-1
+            )
         if self.lstm_use_prev_reward:
-            self.view_requirements[SampleBatch.PREV_REWARDS] = \
-                ViewRequirement(SampleBatch.REWARDS, shift=-1)
+            self.view_requirements[SampleBatch.PREV_REWARDS] = ViewRequirement(
+                SampleBatch.REWARDS, shift=-1
+            )
 
         # Internal states view requirements.
         for i in range(2):
-            space = Box(-1.0, 1.0, shape=(self.lstm_cell_size, ))
-            self.view_requirements["state_in_{}".format(i)] = \
-                ViewRequirement(
-                    "state_out_{}".format(i),
-                    shift=-1,
-                    used_for_compute_actions=True,
-                    batch_repeat_value=max_seq_len,
-                    space=space)
-            self.view_requirements["state_out_{}".format(i)] = \
-                ViewRequirement(space=space, used_for_training=True)
+            space = Box(-1.0, 1.0, shape=(self.lstm_cell_size,))
+            self.view_requirements["state_in_{}".format(i)] = ViewRequirement(
+                "state_out_{}".format(i),
+                shift=-1,
+                used_for_compute_actions=True,
+                batch_repeat_value=max_seq_len,
+                space=space,
+            )
+            self.view_requirements["state_out_{}".format(i)] = ViewRequirement(
+                space=space, used_for_training=True
+            )
 
-    def call(self, input_dict: SampleBatch) -> \
-            (TensorType, List[TensorType], Dict[str, TensorType]):
+    def call(
+        self, input_dict: SampleBatch
+    ) -> (TensorType, List[TensorType], Dict[str, TensorType]):
         assert input_dict.get(SampleBatch.SEQ_LENS) is not None
         # Push obs through underlying (wrapped) model first.
         wrapped_out, _, _ = self.wrapped_keras_model(input_dict)
@@ -405,26 +421,37 @@ class Keras_LSTMWrapper(tf.keras.Model if tf else object):
             if isinstance(self.action_space, (Discrete, MultiDiscrete)):
                 prev_a = one_hot(prev_a, self.action_space)
             prev_a_r.append(
-                tf.reshape(tf.cast(prev_a, tf.float32), [-1, self.action_dim]))
+                tf.reshape(tf.cast(prev_a, tf.float32), [-1, self.action_dim])
+            )
         if self.lstm_use_prev_reward:
             prev_a_r.append(
                 tf.reshape(
-                    tf.cast(input_dict[SampleBatch.PREV_REWARDS], tf.float32),
-                    [-1, 1]))
+                    tf.cast(input_dict[SampleBatch.PREV_REWARDS], tf.float32), [-1, 1]
+                )
+            )
 
         if prev_a_r:
             wrapped_out = tf.concat([wrapped_out] + prev_a_r, axis=1)
 
-        max_seq_len = tf.shape(wrapped_out)[0] // tf.shape(
-            input_dict[SampleBatch.SEQ_LENS])[0]
+        max_seq_len = (
+            tf.shape(wrapped_out)[0] // tf.shape(input_dict[SampleBatch.SEQ_LENS])[0]
+        )
         wrapped_out_plus_time_dim = add_time_dimension(
-            wrapped_out, max_seq_len=max_seq_len, framework="tf")
-        model_out, value_out, h, c = self._rnn_model([
-            wrapped_out_plus_time_dim, input_dict[SampleBatch.SEQ_LENS],
-            input_dict["state_in_0"], input_dict["state_in_1"]
-        ])
+            wrapped_out, max_seq_len=max_seq_len, framework="tf"
+        )
+        model_out, value_out, h, c = self._rnn_model(
+            [
+                wrapped_out_plus_time_dim,
+                input_dict[SampleBatch.SEQ_LENS],
+                input_dict["state_in_0"],
+                input_dict["state_in_1"],
+            ]
+        )
         model_out_no_time_dim = tf.reshape(
-            model_out, tf.concat([[-1], tf.shape(model_out)[2:]], axis=0))
-        return model_out_no_time_dim, [h, c], {
-            SampleBatch.VF_PREDS: tf.reshape(value_out, [-1])
-        }
+            model_out, tf.concat([[-1], tf.shape(model_out)[2:]], axis=0)
+        )
+        return (
+            model_out_no_time_dim,
+            [h, c],
+            {SampleBatch.VF_PREDS: tf.reshape(value_out, [-1])},
+        )
