@@ -27,9 +27,8 @@ def get_best_model_checkpoint(analysis):
             _, _, raw_model = pickle.load(inputFile)
         best_bst.load_model(bytearray(raw_model))
     except IsADirectoryError:
-        best_bst.load_model(
-            os.path.join(analysis.best_checkpoint, CHECKPOINT_FILENAME))
-    accuracy = 1. - analysis.best_result["eval-logloss"]
+        best_bst.load_model(os.path.join(analysis.best_checkpoint, CHECKPOINT_FILENAME))
+    accuracy = 1.0 - analysis.best_result["eval-logloss"]
     print(f"Best model parameters: {analysis.best_config}")
     print(f"Best model total accuracy: {accuracy:.4f}")
     return best_bst
@@ -45,8 +44,7 @@ def train_breast_cancer(config: dict, checkpoint_dir=None):
     # Load dataset
     data, labels = sklearn.datasets.load_breast_cancer(return_X_y=True)
     # Split into train and test set
-    train_x, test_x, train_y, test_y = train_test_split(
-        data, labels, test_size=0.25)
+    train_x, test_x, train_y, test_y = train_test_split(data, labels, test_size=0.25)
     # Build input matrices for XGBoost
     train_set = xgb.DMatrix(train_x, label=train_y)
     test_set = xgb.DMatrix(test_x, label=test_y)
@@ -74,8 +72,10 @@ def train_breast_cancer(config: dict, checkpoint_dir=None):
                 filename=CHECKPOINT_FILENAME,
                 # checkpointing should happen every iteration
                 # with dynamic resource allocation
-                frequency=1)
-        ])
+                frequency=1,
+            )
+        ],
+    )
 
 
 # TRAINABLE (CLASS) API EXAMPLE
@@ -83,13 +83,13 @@ class BreastCancerTrainable(Trainable):
     def setup(self, config):
         self.config = config
         self.nthread = config.pop("nthread", 1)
-        self.new_nthread = None
         self.model: xgb.Booster = None
         # Load dataset
         data, labels = sklearn.datasets.load_breast_cancer(return_X_y=True)
         # Split into train and test set
         train_x, test_x, train_y, test_y = train_test_split(
-            data, labels, test_size=0.25)
+            data, labels, test_size=0.25
+        )
         # Build input matrices for XGBoost
         self.train_set = xgb.DMatrix(train_x, label=train_y)
         self.test_set = xgb.DMatrix(test_x, label=test_y)
@@ -97,8 +97,10 @@ class BreastCancerTrainable(Trainable):
     def step(self):
         # you can also obtain current trial resources:
         current_resources = self.trial_resources
-        # testing purposes only:
-        assert int(current_resources.head_cpus) == int(self.nthread)
+        if isinstance(current_resources, PlacementGroupFactory):
+            self.nthread = current_resources.head_cpus
+        else:
+            self.nthread = current_resources.cpu
 
         results = {}
         config = self.config.copy()
@@ -110,44 +112,31 @@ class BreastCancerTrainable(Trainable):
             verbose_eval=False,
             xgb_model=self.model,
             evals_result=results,
-            num_boost_round=1)
+            num_boost_round=1,
+        )
         print(config, results)
-        return {
-            "eval-logloss": results["eval"]["logloss"][-1],
-            "nthread": self.nthread
-        }
+        return {"eval-logloss": results["eval"]["logloss"][-1], "nthread": self.nthread}
 
     def save_checkpoint(self, checkpoint_dir):
         path = os.path.join(checkpoint_dir, "checkpoint")
         with open(path, "wb") as outputFile:
-            pickle.dump((self.config, self.nthread, self.model.save_raw()),
-                        outputFile)
+            pickle.dump((self.config, self.nthread, self.model.save_raw()), outputFile)
         return path
 
     def load_checkpoint(self, checkpoint_path):
         with open(checkpoint_path, "rb") as inputFile:
             self.config, self.nthread, raw_model = pickle.load(inputFile)
-        if self.new_nthread:
-            self.nthread = self.new_nthread
-            self.new_nthread = None
         self.model = Booster()
         self.model.load_model(bytearray(raw_model))
         data, labels = sklearn.datasets.load_breast_cancer(return_X_y=True)
         # Split into train and test set
         train_x, test_x, train_y, test_y = train_test_split(
-            data, labels, test_size=0.25)
+            data, labels, test_size=0.25
+        )
         # Build input matrices for XGBoost
         self.train_set = xgb.DMatrix(train_x, label=train_y)
         self.test_set = xgb.DMatrix(test_x, label=test_y)
 
-    def update_resources(
-            self, new_resources: Union[PlacementGroupFactory, Resources]):
-        # this is called before `load_checkpoint`
-        if isinstance(new_resources, PlacementGroupFactory):
-            self.new_nthread = new_resources.head_cpus
-        else:
-            self.new_nthread = new_resources.cpu
-
 
 def tune_xgboost(use_class_trainable=True):
     search_space = {
@@ -164,13 +153,14 @@ def tune_xgboost(use_class_trainable=True):
     }
     # This will enable aggressive early stopping of bad trials.
     base_scheduler = ASHAScheduler(
-        max_t=16,  # 16 training iterations
-        grace_period=1,
-        reduction_factor=2)
+        max_t=16, grace_period=1, reduction_factor=2  # 16 training iterations
+    )
 
     def example_resources_allocation_function(
-            trial_runner: "trial_runner.TrialRunner", trial: Trial,
-            result: Dict[str, Any], scheduler: "ResourceChangingScheduler"
+        trial_runner: "trial_runner.TrialRunner",
+        trial: Trial,
+        result: Dict[str, Any],
+        scheduler: "ResourceChangingScheduler",
     ) -> Union[None, PlacementGroupFactory, Resources]:
         """This is a basic example of a resource allocating function.
 
@@ -182,7 +172,7 @@ def tune_xgboost(use_class_trainable=True):
         trial has currently, the scheduler will skip the update process
         internally (same with None).
 
-        See :func:`evenly_distribute_cpus_gpus` for a more complex,
+        See :class:`DistributeResources` for a more complex,
         robust approach.
 
         Args:
@@ -212,26 +202,27 @@ def tune_xgboost(use_class_trainable=True):
 
         # Get the number of CPUs available in total (not just free)
         total_available_cpus = (
-            trial_runner.trial_executor._avail_resources.cpu)
+            trial_runner.trial_executor._resource_updater.get_num_cpus()
+        )
 
         # Divide the free CPUs among all live trials
         cpu_to_use = max(
-            min_cpu,
-            total_available_cpus // len(trial_runner.get_live_trials()))
+            min_cpu, total_available_cpus // len(trial_runner.get_live_trials())
+        )
 
         # Assign new CPUs to the trial in a PlacementGroupFactory
         return PlacementGroupFactory([{"CPU": cpu_to_use, "GPU": 0}])
 
     # You can either define your own resources_allocation_function, or
-    # use the default one - evenly_distribute_cpus_gpus
+    # use the default one - DistributeResources
 
     # from ray.tune.schedulers.resource_changing_scheduler import \
-    #    evenly_distribute_cpus_gpus
+    #    DistributeResources
 
     scheduler = ResourceChangingScheduler(
         base_scheduler=base_scheduler,
         resources_allocation_function=example_resources_allocation_function
-        # resources_allocation_function=evenly_distribute_cpus_gpus  # default
+        # resources_allocation_function=DistributeResources()  # default
     )
 
     if use_class_trainable:
@@ -243,14 +234,12 @@ def tune_xgboost(use_class_trainable=True):
         fn,
         metric="eval-logloss",
         mode="min",
-        resources_per_trial=PlacementGroupFactory([{
-            "CPU": 1,
-            "GPU": 0
-        }]),
+        resources_per_trial=PlacementGroupFactory([{"CPU": 1, "GPU": 0}]),
         config=search_space,
         num_samples=1,
         scheduler=scheduler,
-        checkpoint_at_end=use_class_trainable)
+        checkpoint_at_end=use_class_trainable,
+    )
 
     if use_class_trainable:
         assert analysis.results_df["nthread"].max() > 1
@@ -260,24 +249,27 @@ def tune_xgboost(use_class_trainable=True):
 
 if __name__ == "__main__":
     import argparse
+
     parser = argparse.ArgumentParser()
     parser.add_argument(
         "--server-address",
         type=str,
         default=None,
         required=False,
-        help="The address of server to connect to if using "
-        "Ray Client.")
+        help="The address of server to connect to if using Ray Client.",
+    )
     parser.add_argument(
         "--class-trainable",
         action="store_true",
         default=False,
-        help="set to use the Trainable (class) API instead of functional one")
+        help="set to use the Trainable (class) API instead of functional one",
+    )
     parser.add_argument(
         "--test",
         action="store_true",
         default=False,
-        help="set to run both functional and Trainable APIs")
+        help="set to run both functional and Trainable APIs",
+    )
     args, _ = parser.parse_known_args()
 
     if args.server_address:
@@ -298,8 +290,8 @@ if __name__ == "__main__":
         # We have to make sure it gets executed on the same node that
         # ``tune.run`` is called on.
         from ray.util.ml_utils.node import force_on_current_node
-        remote_fn = force_on_current_node(
-            ray.remote(get_best_model_checkpoint))
+
+        remote_fn = force_on_current_node(ray.remote(get_best_model_checkpoint))
         best_bst = ray.get(remote_fn.remote(analysis))
     else:
         best_bst = get_best_model_checkpoint(analysis)
