@@ -12,13 +12,18 @@ from ray import ObjectRef, logger
 from ray.actor import ActorClass
 from ray.tune.resources import Resources
 from ray.util.annotations import PublicAPI, DeveloperAPI
-from ray.util.placement_group import PlacementGroup, get_placement_group, \
-    placement_group, placement_group_table, remove_placement_group
+from ray.util.placement_group import (
+    PlacementGroup,
+    get_placement_group,
+    placement_group,
+    placement_group_table,
+    remove_placement_group,
+)
 
 if TYPE_CHECKING:
     from ray.tune.trial import Trial
 
-TUNE_PLACEMENT_GROUP_REMOVAL_DELAY = 2.
+TUNE_PLACEMENT_GROUP_REMOVAL_DELAY = 2.0
 
 _tune_pg_prefix = None
 
@@ -121,23 +126,28 @@ class PlacementGroupFactory:
 
     """
 
-    def __init__(self,
-                 bundles: List[Dict[str, Union[int, float]]],
-                 strategy: str = "PACK",
-                 *args,
-                 **kwargs):
-        assert len(bundles) > 0, (
-            "Cannot initialize a PlacementGroupFactory with zero bundles.")
-        if not bundles[0]:
+    def __init__(
+        self,
+        bundles: List[Dict[str, Union[int, float]]],
+        strategy: str = "PACK",
+        *args,
+        **kwargs,
+    ):
+        assert (
+            len(bundles) > 0
+        ), "Cannot initialize a PlacementGroupFactory with zero bundles."
+
+        self._bundles = [
+            {k: float(v) for k, v in bundle.items() if v != 0} for bundle in bundles
+        ]
+
+        if not self._bundles[0]:
             # This is when trainable itself doesn't need resources.
             self._head_bundle_is_empty = True
-            bundles.pop(0)
+            self._bundles.pop(0)
         else:
             self._head_bundle_is_empty = False
 
-        self._bundles = [{k: float(v)
-                          for k, v in bundle.items() if v != 0}
-                         for bundle in bundles]
         self._strategy = strategy
         self._args = args
         self._kwargs = kwargs
@@ -159,8 +169,7 @@ class PlacementGroupFactory:
     @property
     @DeveloperAPI
     def head_cpus(self) -> float:
-        return 0.0 if self._head_bundle_is_empty else self._bundles[0].get(
-            "CPU", 0.0)
+        return 0.0 if self._head_bundle_is_empty else self._bundles[0].get("CPU", 0.0)
 
     @property
     @DeveloperAPI
@@ -180,34 +189,38 @@ class PlacementGroupFactory:
     def _bind(self):
         sig = signature(placement_group)
         try:
-            self._bound = sig.bind(self._bundles, self._strategy, *self._args,
-                                   **self._kwargs)
+            self._bound = sig.bind(
+                self._bundles, self._strategy, *self._args, **self._kwargs
+            )
         except Exception as exc:
             raise RuntimeError(
                 "Invalid definition for placement group factory. Please check "
                 "that you passed valid arguments to the PlacementGroupFactory "
-                "object.") from exc
+                "object."
+            ) from exc
 
     def __call__(self, *args, **kwargs):
         kwargs.update(self._bound.kwargs)
         # Call with bounded *args and **kwargs
         return placement_group(*self._bound.args, **kwargs)
 
-    def __eq__(self, other):
-        return self._bound == other._bound
+    def __eq__(self, other: "PlacementGroupFactory"):
+        return (
+            self._bound == other._bound
+            and self.head_bundle_is_empty == other.head_bundle_is_empty
+        )
 
     def __hash__(self):
         if not self._hash:
             # Cache hash
             self._hash = hash(
                 json.dumps(
-                    {
-                        "args": self._bound.args,
-                        "kwargs": self._bound.kwargs
-                    },
+                    {"args": self._bound.args, "kwargs": self._bound.kwargs},
                     sort_keys=True,
                     indent=0,
-                    ensure_ascii=True))
+                    ensure_ascii=True,
+                )
+            )
         return self._hash
 
     def __getstate__(self):
@@ -232,26 +245,30 @@ def resource_dict_to_pg_factory(spec: Optional[Dict[str, float]]):
     spec = spec.copy()
     extra_custom = spec.pop("extra_custom_resources", {}) or {}
 
-    if any(k.startswith("extra_") and spec[k]
-           for k in spec) or any(extra_custom[k] for k in extra_custom):
+    if any(k.startswith("extra_") and spec[k] for k in spec) or any(
+        extra_custom[k] for k in extra_custom
+    ):
         raise ValueError(
             "Passing `extra_*` resource requirements to `resources_per_trial` "
             "is deprecated. Please use a `PlacementGroupFactory` object "
-            "to define your resource requirements instead.")
+            "to define your resource requirements instead."
+        )
 
-    cpus = spec.pop("cpu", 0.)
-    gpus = spec.pop("gpu", 0.)
-    memory = spec.pop("memory", 0.)
-    object_store_memory = spec.pop("object_store_memory", 0.)
+    cpus = spec.pop("cpu", 0.0)
+    gpus = spec.pop("gpu", 0.0)
+    memory = spec.pop("memory", 0.0)
+    object_store_memory = spec.pop("object_store_memory", 0.0)
 
     bundle = {k: v for k, v in spec.pop("custom_resources", {}).items()}
 
-    bundle.update({
-        "CPU": cpus,
-        "GPU": gpus,
-        "memory": memory,
-        "object_store_memory": object_store_memory
-    })
+    bundle.update(
+        {
+            "CPU": cpus,
+            "GPU": gpus,
+            "memory": memory,
+            "object_store_memory": object_store_memory,
+        }
+    )
 
     return PlacementGroupFactory([bundle])
 
@@ -277,19 +294,21 @@ class PlacementGroupManager:
         self._prefix = prefix
 
         # Sets of staged placement groups by factory
-        self._staging: Dict[PlacementGroupFactory, Set[
-            PlacementGroup]] = defaultdict(set)
+        self._staging: Dict[PlacementGroupFactory, Set[PlacementGroup]] = defaultdict(
+            set
+        )
         # Sets of ready and unused placement groups by factory
-        self._ready: Dict[PlacementGroupFactory, Set[
-            PlacementGroup]] = defaultdict(set)
+        self._ready: Dict[PlacementGroupFactory, Set[PlacementGroup]] = defaultdict(set)
         # Ray futures to check if a placement group is ready
-        self._staging_futures: Dict[ObjectRef, Tuple[PlacementGroupFactory,
-                                                     PlacementGroup]] = {}
+        self._staging_futures: Dict[
+            ObjectRef, Tuple[PlacementGroupFactory, PlacementGroup]
+        ] = {}
 
         # Cache of unstaged PGs (cleaned after full PG removal)
         self._unstaged_pg_pgf: Dict[PlacementGroup, PlacementGroupFactory] = {}
-        self._unstaged_pgf_pg: Dict[PlacementGroupFactory, Set[
-            PlacementGroup]] = defaultdict(set)
+        self._unstaged_pgf_pg: Dict[
+            PlacementGroupFactory, Set[PlacementGroup]
+        ] = defaultdict(set)
 
         # Placement groups used by trials
         self._in_use_pgs: Dict[PlacementGroup, "Trial"] = {}
@@ -300,17 +319,16 @@ class PlacementGroupManager:
         self._cached_pgs: Dict[PlacementGroup, PlacementGroupFactory] = {}
 
         # Placement groups scheduled for delayed removal.
+        # This is used as a damper to filter out some high frequency change
+        # in resources request.
+        # Only PGs that have never been used go here.
+        # TODO(xwjiang): `self._pgs_for_removal` and `self._unstaged_xxx`
+        #  are really the same now. We should consolidate to using one.
+        #  Also `remove_placement_group` method should just be combined with
+        #  `unstage_unused_xxx`.
         self._pgs_for_removal: Dict[PlacementGroup, float] = {}
         self._removal_delay = TUNE_PLACEMENT_GROUP_REMOVAL_DELAY
 
-        # Latest PG staging time to check if still in grace period.
-        self._latest_staging_start_time = time.time()
-
-        # Seconds we wait for a trial to come up before we make blocking calls
-        # to process events
-        self._grace_period = float(
-            os.getenv("TUNE_TRIAL_STARTUP_GRACE_PERIOD", 10.))
-
         self._max_staging = max_staging
 
     def set_max_staging(self, max_staging: int):
@@ -340,8 +358,10 @@ class PlacementGroupManager:
         """
         # Wrap in list so we can modify the dict
         for pg in list(self._pgs_for_removal):
-            if force or (time.time() -
-                         self._removal_delay) >= self._pgs_for_removal[pg]:
+            if (
+                force
+                or (time.time() - self._removal_delay) >= self._pgs_for_removal[pg]
+            ):
                 self._pgs_for_removal.pop(pg)
 
                 remove_placement_group(pg)
@@ -366,7 +386,8 @@ class PlacementGroupManager:
                 shut down.
         """
         should_cleanup = not int(
-            os.getenv("TUNE_PLACEMENT_GROUP_CLEANUP_DISABLED", "0"))
+            os.getenv("TUNE_PLACEMENT_GROUP_CLEANUP_DISABLED", "0")
+        )
         if should_cleanup:
             has_non_removed_pg_left = True
             while has_non_removed_pg_left:
@@ -422,7 +443,6 @@ class PlacementGroupManager:
 
         self._staging[pgf].add(pg)
         self._staging_futures[pg.ready()] = (pgf, pg)
-        self._latest_staging_start_time = time.time()
 
         return True
 
@@ -443,13 +463,20 @@ class PlacementGroupManager:
             ready, _ = ray.wait(list(self._staging_futures.keys()), timeout=0)
 
             for ready_fut in ready:
-                ready_pgf, ready_pg = self._staging_futures.pop(ready_fut)
+                self.handle_ready_future(ready_fut)
 
-                self._staging[ready_pgf].remove(ready_pg)
-                self._ready[ready_pgf].add(ready_pg)
+    def handle_ready_future(self, ready_fut):
+        ready_pgf, ready_pg = self._staging_futures.pop(ready_fut)
 
-    def get_full_actor_cls(self, trial: "Trial",
-                           actor_cls: ActorClass) -> Optional[ActorClass]:
+        self._staging[ready_pgf].remove(ready_pg)
+        self._ready[ready_pgf].add(ready_pg)
+
+    def get_staging_future_list(self):
+        return list(self._staging_futures.keys())
+
+    def get_full_actor_cls(
+        self, trial: "Trial", actor_cls: ActorClass
+    ) -> Optional[ActorClass]:
         """Get a fully configured actor class.
 
         Returns the actor handle if the placement group is ready. In this case,
@@ -490,14 +517,16 @@ class PlacementGroupManager:
                 placement_group_capture_child_tasks=True,
                 num_cpus=num_cpus,
                 num_gpus=num_gpus,
-                resources=resources)
+                resources=resources,
+            )
         else:
             return actor_cls.options(
                 placement_group=pg,
                 placement_group_capture_child_tasks=True,
                 num_cpus=0,
                 num_gpus=0,
-                resources={})
+                resources={},
+            )
 
     def has_ready(self, trial: "Trial", update: bool = False) -> bool:
         """Return True if placement group for trial is ready.
@@ -566,8 +595,10 @@ class PlacementGroupManager:
         pgf = self._cached_pgs.pop(pg)
         trial_pgf = trial.placement_group_factory
 
-        assert pgf == trial_pgf, f"Cannot assign placement group with a " \
-                                 f"non-matching factory to trial {trial}"
+        assert pgf == trial_pgf, (
+            f"Cannot assign placement group with a "
+            f"non-matching factory to trial {trial}"
+        )
 
         logger.debug(f"For trial {trial} RE-use pg {pg.id}")
 
@@ -579,68 +610,21 @@ class PlacementGroupManager:
     def clean_cached_pg(self, pg: PlacementGroup):
         self._cached_pgs.pop(pg)
 
-    def return_or_clean_cached_pg(self, pg: PlacementGroup):
-        """Return cached pg, making it available for other trials to use.
-
-        This will try to replace another staged placement group. If this
-        is unsuccessful, destroy the placement group instead.
-
-        Args:
-            pg (PlacementGroup): Return this cached placement group.
-
-        Returns:
-            Boolean indicating if the placement group was returned (True)
-                or destroyed (False)
-        """
-        pgf = self._cached_pgs.pop(pg)
-
-        # Replace staged placement group
-        staged_pg = self._unstage_unused_pg(pgf)
-
-        # Could not replace
-        if not staged_pg:
-            self.remove_pg(pg)
-            return False
-
-        # Replace successful
-        self.remove_pg(staged_pg)
-        self._ready[pgf].add(pg)
-        return True
-
-    def return_pg(self,
-                  trial: "Trial",
-                  destroy_pg_if_cannot_replace: bool = True):
-        """Return pg, making it available for other trials to use.
-
-        If destroy_pg_if_cannot_replace is True, this will only return
-        a placement group if a staged placement group can be replaced
-        by it. If not, it will destroy the placement group.
+    def remove_from_in_use(self, trial: "Trial") -> PlacementGroup:
+        """Return pg back to Core scheduling.
 
         Args:
             trial (Trial): Return placement group of this trial.
-
-        Returns:
-            Boolean indicating if the placement group was returned.
         """
-        pgf = trial.placement_group_factory
+
         pg = self._in_use_trials.pop(trial)
         self._in_use_pgs.pop(pg)
 
-        if destroy_pg_if_cannot_replace:
-            staged_pg = self._unstage_unused_pg(pgf)
-
-            # Could not replace
-            if not staged_pg:
-                self.remove_pg(pg)
-                return False
-
-            self.remove_pg(staged_pg)
-        self._ready[pgf].add(pg)
-
-        return True
+        return pg
 
     def _unstage_unused_pg(
-            self, pgf: PlacementGroupFactory) -> Optional[PlacementGroup]:
+        self, pgf: PlacementGroupFactory
+    ) -> Optional[PlacementGroup]:
         """Unstage an unsued (i.e. staging or ready) placement group.
 
         This method will find an unused placement group and remove it from
@@ -688,10 +672,6 @@ class PlacementGroupManager:
 
         return trial_pg
 
-    def in_staging_grace_period(self):
-        return self._staging_futures and self._grace_period and time.time(
-        ) <= self._latest_staging_start_time + self._grace_period
-
     def reconcile_placement_groups(self, trials: List["Trial"]):
         """Reconcile placement groups to match requirements.
 
@@ -717,8 +697,9 @@ class PlacementGroupManager:
             if trial in self._in_use_trials:
                 current_counts[trial.placement_group_factory] += 1
 
-            pgf_expected[trial.placement_group_factory] += \
+            pgf_expected[trial.placement_group_factory] += (
                 1 if trial.status in ["PAUSED", "PENDING", "RUNNING"] else 0
+            )
 
         # Ensure that unexpected placement groups are accounted for
         for pgf in self._staging:
@@ -750,8 +731,10 @@ class PlacementGroupManager:
             while expected > current_counts[pgf]:
                 self._stage_pgf_pg(pgf)
                 current_counts[pgf] += 1
-                logger.debug(f"Adding an expected but previously unstaged "
-                             f"placement group for factory {pgf}")
+                logger.debug(
+                    f"Adding an expected but previously unstaged "
+                    f"placement group for factory {pgf}"
+                )
 
     def occupied_resources(self):
         """Return a dictionary of currently in-use resources."""
